package it.unibo.tw.dao;

import java.util.logging.Logger;

public abstract class DAOFactory {

	static Logger logger = Logger.getLogger(DAOFactory.class.getCanonicalName());

	// --- Actual factory method ---
	
	/**
	 * Depending on the input parameter
	 * this method returns one out of several possible 
	 * implementations of this factory spec 
	 */
	public static DAOFactory getDAOFactory(String concreteDaoFactoryClassName) {
		try {
			return (DAOFactory) Class.forName(concreteDaoFactoryClassName).newInstance();
		}
		catch (Exception e) {
			logger.warning(e.getMessage());
			e.printStackTrace();
			return null;
		}
	}
	
	// --- Factory specification: concrete factories implementing this spec must provide this methods! ---

	/**
	 * Method to obtain a DATA ACCESS OBJECT
	 * for the datatype 'Feed'
	 */
	public abstract FeedDAO getFeedDAO();

}
