package it.unibo.tw.dao;

import it.unibo.tw.dao.hsqldb.HsqldbDAOFactory;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class FeedDAOTest {

	public static final String DAO = HsqldbDAOFactory.class.getCanonicalName();
	
	public FeedDAO feedDAO = null;

	public FeedDTO feedDTO = null;

	@Before
	public void createTable() {
		feedDAO = DAOFactory.getDAOFactory(DAO).getFeedDAO();
		feedDAO.createTable();
		//
		feedDTO = new FeedDTO("titolo","descrizione","autore","prova","oggi","http://www.unibo.it","http://www.unibo.it/logo.gif");
	}

	@After
	public void dropTable() {
		feedDAO.dropTable();
		feedDAO = null;
		feedDTO = null;
	}

	// --------

	@Test
	public void testReadNone() {
		Assert.assertEquals("No entries in the beginning",0,feedDAO.find(null).size());
	}

	@Test
	public void testCreateAndRead() {
		Long FeedDTO_id = feedDAO.create(feedDTO);
		Assert.assertTrue("Created entry got non negative id",FeedDTO_id >= 0);
		Assert.assertTrue("Created entries are re-read correctly",feedDTO.getTitle().equals(feedDAO.read(FeedDTO_id).getTitle()));
	}

	@Test
	public void testFind() {
		Long FeedDTO_id = feedDAO.create(feedDTO);
		Assert.assertTrue("Created entry got non negative id",FeedDTO_id >= 0);
		Assert.assertEquals("One entry was written to the database",1,feedDAO.find(null).size());
		Assert.assertEquals("One entry with category '"+feedDTO.getCategory()+"' was written to the database",1,feedDAO.find(feedDTO.getCategory()).size());
		Assert.assertEquals("No entries with category 'aaa' were written to the database",0,feedDAO.find("aaa").size());
	}

	@Test
	public void testDelete() {
		Long FeedDTO_id = feedDAO.create(feedDTO);
		// la delete accetta un intero DTO, non solo l'id! quindi dobbiamo recuperare il DTO intero prima!
		FeedDTO FeedDTO_to_delete = feedDAO.read(FeedDTO_id);
		Assert.assertTrue("Correct deletion returns boolean result true",feedDAO.delete(FeedDTO_to_delete.getId()));
		Assert.assertEquals("No feeds left",0,feedDAO.find(null).size());
	}

}
